# -*- coding: utf-8 -*-

import requests
import json
import datetime
import os
from tgasdk.sdk import *
import pdb
import pymysql
from mysql.mmysql import MysqlBase


# 按提供的gameid从tga数据库中查询并输出其对应的url,suffix,appid,api_key，tgaid以dict形式存放
class GetTgaConfig():
    def __init__(self):
        self.url = "http://10.10.3.17:8992/querySql"
        self.TGA = {'user': 'mytga', 'pswd': 'gzVwh4HGR68G', 'host': '10.10.3.5', 'db': 'tga'}

    def get_api_key(self, gameid, channel=6001):
        item = {}
        item['url'] = self.url
        sql = "select suffix,appid,api_secret,tgaid from tgainfo where gameid={} and channelid={} and " \
              "in_used=1;".format(gameid, channel)
        t = MysqlBase(**self.TGA)
        data = t.query(sql)
        if data:
            item['suffix'], item['appid'], item['api_key'], item['tgaid'] = data[0]
        return item

class FromTga:
    def __init__(self, url, token):
        if not token:
            raise Exception("{0} token not found in env !")
        self.url = url
        self.token = token
        self.output = "/data/logs/tga-report/"

    def get_data(self, sql):
        data = {'token': self.token, 'sql': sql}
        r = requests.post(self.url, data)
        if r.status_code != requests.codes.ok:
            print("connect tga failed!")
            return None

        out = r.content.decode('utf-8')
        if json.loads(out.split('\r\n')[0]).get('return_code', None) != 0:
            # raise Exception("get data from tga failed!")
            print("get data from tga failed!,sql was {}".format(sql))
            return None

        data_out = out.split('\r\n')[1:]
        output = list()
        for row in data_out:
            if row:
                try:
                    output.append(json.loads(row))
                except Exception as e:
                    print("转化数据失败,{} 提示为{}".format(row, e))
        return output

    def init_tga_write(self, tgaid):
        '''
        from tgasdk.sdk import TGAnalytics, BatchConsumer, LoggingConsumer, AsyncBatchConsumer
        也可引入TGAnalytics与指定的Consumer
        '''
        # now = datetime.date.today().strftime('%Y%m%d%H%M%S')
        # # 初始化SDK
        # filename = "{}/report_{}_{}.log".format(self.output, project, now)
        # 检查目录是否存在，如不存在创建之
        paths = "{}/{}".format(self.output, tgaid)
        if not os.path.isdir(paths):
            os.makedirs(paths, mode=0o755)

        self.tga = TGAnalytics(LoggingConsumer(paths))

    def _close_tga(self, tga):
        self.tga.flush()
        self.tga.close()

    def _split_user_data(self, data):
        distinct_id = data.get('distinct_id', None)
        account_id = data.get('account_id', None)

        if not (distinct_id or account_id):
            print("distinct_id 或 account_id 必须有一则有值！")
            return None

        if distinct_id:
            data.pop('distinct_id')
        if account_id:
            data.pop('account_id')

        return distinct_id, account_id, data

    def put_event_data(self, data, event_name="Payment"):
        # tga = self._init_tga_write()
        try:
            distinct_id, account_id, new_data = self._split_user_data(data)
        except Exception as e:
            print("拆解数据错误，输出为{}.请检查！".format(e))
            return False

        # properties = {
        #     "#time": datetime.datetime.now(),
        #     "#ip": "192.168.1.1",
        #     "Product_Name": "月卡",
        #     "Price": 30,
        #     "OrderId": "abc_123"
        # }

        # 上传事件，包含账号ID与访客ID
        try:
            self.tga.track(distinct_id, account_id, event_name, new_data)
        except Exception as e:
            print("write to tga failed,output was {}".format(e))
            # self._close_tga(tga)
            return False
        # finally:
        #     self._close_tga(tga)
        return True

    def put_user_data(self, data, method='user_set'):
        # tga = self._init_tga_write()
        try:
            distinct_id, account_id, new_data = self._split_user_data(data)
        except Exception as e:
            print("拆解数据错误，输出为{}.请检查！".format(e))
            return False
        try:
            if method.lower() == "user_set":
                self.tga.user_set(account_id=account_id, distinct_id=distinct_id, properties=new_data)
            elif method.lower() == "user_setonce":
                self.tga.user_setOnce(account_id=account_id, distinct_id=distinct_id, properties=new_data)
            elif method.lower() == "user_add":
                self.tga.user_add(account_id=account_id, distinct_id=distinct_id, properties=new_data)
            elif method.lower() == "user_del":
                self.tga.user_del(account_id=account_id, distinct_id=distinct_id)
            else:
                print("请提供用户操作类型  [user_set/user_setOnce/user_add/user_del] !")
                return False
        except Exception as e:
            print("write to tga failed,output was {}".format(e))
            return False
        # finally:
        #     self._close_tga(tga)
        return True
